function [U2_filt2,V2_filt2,W12_filt2,W22_filt2] = ADV_Preprocessing(U2,V2,W12,W22,tVEL2,Cor2)

figSwitchPP = 0; % if 1, figures are shown; if 0, figures are not shown
CorTR = 70; %Treshold of correlation (%)
NaNTR = 50; %Treshold of NaN values (%) above which measured cells will be removed
interp_method = 'pchip'; %'linear','pchip'(partial cubic hermite interpolation),'spline'(cubic spline)
n_vertcell = 1; %number of vertical cell for plotting velocities
n_spectra = [];
hx = 0.01;
hy = 0.01;

%% Calculate mean and standard deviation of velocity components

BasicStatsUnfiltered = struct('VelMean',[],'VelStd',[]);
for i=1:length(U2)
    for n=1:size(U2{1},2)
        BasicStatsUnfiltered.VelMean(i,n,1)=mean(U2{i}(:,n),'omitnan');
        BasicStatsUnfiltered.VelStd(i,n,1)=std(U2{i}(:,n),'omitnan');
        if ~isempty(V2)
            BasicStatsUnfiltered.VelMean(i,n,2)=mean(V2{i}(:,n),'omitnan');
            BasicStatsUnfiltered.VelStd(i,n,2)=std(V2{i}(:,n),'omitnan');
        end
        if ~isempty(W12)
            BasicStatsUnfiltered.VelMean(i,n,3)=mean(W12{i}(:,n),'omitnan');
            BasicStatsUnfiltered.VelStd(i,n,3)=std(W12{i}(:,n),'omitnan');
        end
        if ~isempty(W22)
            BasicStatsUnfiltered.VelMean(i,n,4)=mean(W22{i}(:,n),'omitnan');
            BasicStatsUnfiltered.VelStd(i,n,4)=std(W22{i}(:,n),'omitnan');
        end
    end
end

%% Plot mean and standard deviation of velocities
if figSwitchPP==1
    figure;
    subplot(2,1,1)
    hold on;
    errorbar(BasicStatsUnfiltered.VelMean(:,1,1),BasicStatsUnfiltered.VelStd(:,1,1),'o');
    if ~isempty(V2)
        errorbar(BasicStatsUnfiltered.VelMean(:,1,2),BasicStatsUnfiltered.VelStd(:,1,2),'x');
        legend('u','v');
    else
        legend('u');
    end
    plot(xlim,[0 0],'--k');
    hold off;

    if ~isempty(W12) || ~isempty(W22)
        subplot(2,1,2)
        hold on;
        if ~isempty(W12)
            errorbar(BasicStatsUnfiltered.VelMean(:,1,3),BasicStatsUnfiltered.VelStd(:,1,3),'.');
        end
        if ~isempty(W22)
            errorbar(BasicStatsUnfiltered.VelMean(:,1,4),BasicStatsUnfiltered.VelStd(:,1,4),'.');
        end
        if ~isemtpy(W12) && ~isempty(W22)
            legend('w1','w2');
        elseif ~isempty(W12)
            legend('w1');
        else
            legend('w2');
        end
        plot(xlim,[0 0],'--k');
        hold off;
    end
end

%% Show the data for visual inspection of the quality.
if figSwitchPP==1
    figure;
    subplot(4,1,1)
    hold on;
    for i=1:length(U2)
        plot(tVEL2{i},U2{i}(:,n_vertcell))
    end
    plot(xlim,[0 0],'--k');
    xlabel('time')
    ylabel('u (m/s)')
    dateaxis('x',15)
    box on;
    title(sprintf('Vertical cell %d',n_vertcell));
    hold off;

    if ~isempty(V2)
        subplot(4,1,2)
        hold on;
        for i=1:length(V2)
            plot(tVEL2{i},V2{i}(:,n_vertcell))
        end
        plot(xlim,[0 0],'--k');
        xlabel('time')
        ylabel('v (m/s)')
        dateaxis('x',15)
        box on;
        hold off;
    end

    if ~isempty(W12)
        subplot(4,1,3)
        hold on;
        for i=1:length(W12)
            plot(tVEL2{i},W12{i}(:,n_vertcell))
        end
        plot(xlim,[0 0],'--k');
        xlabel('time')
        ylabel('w1 (m/s)')
        dateaxis('x',15)
        box on;
        hold off;
    end

    if ~isempty(W22)
        subplot(4,1,4)
        hold on;
        for i=1:length(W22)
            plot(tVEL2{i},W22{i}(:,n_vertcell))
        end
        plot(xlim,[0 0],'--k');
        xlabel('time')
        ylabel('w2 (m/s)')
        dateaxis('x',15)
        box on;
        hold off;
    end
end

%% Analyse correlation
if figSwitchPP==1
    figure;
    for n=1:4
        subplot(4,1,n)
        hold on;
        for i=1:length(Cor2)
            plot(tVEL2{i},Cor2{i}(:,n_vertcell,n))
        end
        xlabel('time')
        ylabel('correlation')
        dateaxis('x',15)
        box on;
        if n==1
            title(sprintf('Vertical cell %d',n_vertcell));
        end
        hold off;
    end
end

%% Use Bivariate kernel density function for despiking
U2_filt = U2;
Idu = cell(length(U2),size(U2{1},2));
for i=1:length(U2)
    for j=1:size(U2{i},2)
        [U2_filt{i}(:,j), Idu{i,j}] = despikeADV(U2{i}(:,j),hx,hy,i,j,'U2');
    end
end

if ~isempty(V2)
    V2_filt = V2;
    Idv = cell(length(V2),size(V2{1},2));
    for i=1:length(V2)
        for j=1:size(V2{i},2)
            [V2_filt{i}(:,j), Idv{i,j}] = despikeADV(V2{i}(:,j),hx,hy,i,j,'V2');
        end
    end
end

if ~isempty(W12)
    W12_filt = W12;
    Idw1 = cell(length(W12),size(W12{1},2));
    for i=1:length(W12)
        for j=1:size(W12{i},2)
            [W12_filt{i}(:,j), Idw1{i,j}] = despikeADV(W12{i}(:,j),hx,hy,i,j,'W12');
        end
    end
end

if ~isempty(W22)
    W22_filt = W22;
    Idw2 = cell(length(W22),size(W22{1},2));
    for i=1:length(W22)
        for j=1:size(W22{i},2)
            [W22_filt{i}(:,j), Idw2{i,j}] = despikeADV(W22{i}(:,j),hx,hy,i,j,'W22');
        end
    end
end

%%
% Use ID's to detect interpolated values and change to NaN again, so that
% cubic interpolation can be applied. 
for i=1:length(U2)
    for j=1:size(U2{i},2)
        if ~isnan(Idu{i,j})
            U2_filt{i}(Idu{i,j},j) = NaN;
        end
    end
    if ~isempty(V2)
        for j=1:size(V2{i},2)
            if ~isnan(Idv{i,j})
                V2_filt{i}(Idv{i,j},j) = NaN;
            end
        end
    end
    if ~isempty(W12)
        for j=1:size(W12{i},2)
            if ~isnan(Idw1{i,j})
                W12_filt{i}(Idw1{i,j},j) = NaN;
            end
        end
    end
    if ~isempty(W22)
        for j=1:size(W22{i},2)
            if ~isnan(Idw2{i,j})
                W22_filt{i}(Idw2{i,j},j) = NaN;
            end
        end
    end
end

%% Filter the velocities based on corresponding correlations values, with threshold of CorTR (%)
for i=1:length(U2)
    U2_filt{i}(Cor2{i}(:,:,1)<CorTR) = NaN;
    if ~isempty(V2)
        V2_filt{i}(Cor2{i}(:,:,2)<CorTR) = NaN;
    end
    if ~isempty(W12)
        W12_filt{i}(Cor2{i}(:,:,3)<CorTR) = NaN;
    end
    if ~isempty(W22)
        W22_filt{i}(Cor2{i}(:,:,4)<CorTR) = NaN;
    end
end

%Calculate the percentages of NaN after filtering
Percentage_NaN=NaN(length(U2),size(U2{1},2),4);
for i=1:size(Percentage_NaN,1)
    for j=1:size(Percentage_NaN,2)
        Percentage_NaN(i,j,1) = sum(sum(isnan(U2_filt{i}(:,j)),2),1)/numel(U2_filt{i}(:,j));
        if ~isempty(V2)
            Percentage_NaN(i,j,2) = sum(sum(isnan(V2_filt{i}(:,j)),2),1)/numel(V2_filt{i}(:,j));
        end
        if ~isempty(W12)
            Percentage_NaN(i,j,3) = sum(sum(isnan(W12_filt{i}(:,j)),2),1)/numel(W12_filt{i}(:,j));
        end
        if ~isempty(W22)
            Percentage_NaN(i,j,4) = sum(sum(isnan(W22_filt{i}(:,j)),2),1)/numel(W22_filt{i}(:,j));
        end
    end
end

if figSwitchPP==1
    figure;
    h=pcolor([Percentage_NaN(:,:,1) NaN(size(Percentage_NaN,1),1); NaN(1,size(Percentage_NaN,2)+1)]);
    shading flat;
    set(h,'EdgeColor','none');
end

%% Set cells with percentage of NaN > NaNTR to NaN
U2_filt2 = cell(size(U2_filt));
if ~isempty(V2)
    V2_filt2 = U2_filt2;
end
if ~isempty(W12)
    W12_filt2 = U2_filt2;
end
if ~isempty(W22)
    W22_filt2 = U2_filt2;
end

for i=1:length(U2)
    for j=1:size(U2{i},2)
        if any(squeeze(Percentage_NaN(i,j,:))>NaNTR/100)
            U2_filt2{i}(:,j) = NaN(size(U2_filt{i}(:,j)));
            if ~isempty(V2)
                V2_filt2{i}(:,j) = NaN(size(U2_filt{i}(:,j)));
            end
            if ~isempty(W12)
                W12_filt2{i}(:,j) = NaN(size(U2_filt{i}(:,j)));
            end
            if ~isempty(W22)
                W22_filt2{i}(:,j) = NaN(size(U2_filt{i}(:,j)));
            end
        else
            U2_filt2{i}(:,j) = U2_filt{i}(:,j);
            if ~isempty(V2)
                V2_filt2{i}(:,j) = V2_filt{i}(:,j);
            end
            if ~isempty(W12)
                W12_filt2{i}(:,j) = W12_filt{i}(:,j);
            end
            if ~isempty(W22)
                W22_filt2{i}(:,j) = W22_filt{i}(:,j);
            end
        end
    end
end

U2_filt2_noint = U2_filt2;
if ~isempty(V2)
    V2_filt2_noint = V2_filt2;
end
if ~isempty(W12)
    W12_filt2_noint = W12_filt2;
end
if ~isempty(W22)
    W22_filt2_noint = W22_filt2;
end

Percentage_NaN2_noint=NaN(length(U2),size(U2{1},2),4);
for i=1:size(Percentage_NaN2_noint,1)
    for j=1:size(Percentage_NaN2_noint,2)
        Percentage_NaN2_noint(i,j,1) = sum(sum(isnan(U2_filt2_noint{i}(:,j)),2),1)/numel(U2_filt2_noint{i}(:,j));
        if ~isempty(V2)
            Percentage_NaN2_noint(i,j,2) = sum(sum(isnan(V2_filt2_noint{i}(:,j)),2),1)/numel(V2_filt2_noint{i}(:,j));
        end
        if ~isempty(W12)
            Percentage_NaN2_noint(i,j,3) = sum(sum(isnan(W12_filt2_noint{i}(:,j)),2),1)/numel(W12_filt2_noint{i}(:,j));
        end
        if ~isempty(W22)
            Percentage_NaN2_noint(i,j,4) = sum(sum(isnan(W22_filt2_noint{i}(:,j)),2),1)/numel(W22_filt2_noint{i}(:,j));
        end
    end
end

if figSwitchPP==1
    figure;
    h=pcolor([Percentage_NaN2_noint(:,:,1) NaN(size(Percentage_NaN2_noint,1),1); NaN(1,size(Percentage_NaN2_noint,2)+1)]);
    shading flat;
    set(h,'EdgeColor','none');
end

%% Interpolation method - cubic interpolation
for i=1:length(U2)
    for j=1:size(U2{i},2)
        inan = find(~isnan(U2_filt2{i}(:,j)));
        if length(inan)<2
            U2_filt2{i}(:,j) = NaN(size(U2_filt2{i}(:,j)));
        else
            U2_filt2{i}(:,j) = interp1(inan,U2_filt2{i}(inan,j),1:size(U2_filt2{i},1),interp_method,NaN);
        end
        
        if ~isempty(V2)
            inan = find(~isnan(V2_filt2{i}(:,j)));
            if length(inan)<2
                V2_filt2{i}(:,j) = NaN(size(V2_filt2{i}(:,j)));
            else
                V2_filt2{i}(:,j) = interp1(inan,V2_filt2{i}(inan,j),1:size(V2_filt2{i},1),interp_method,NaN);
            end
        end

        if ~isempty(W12)
            inan = find(~isnan(W12_filt2{i}(:,j)));
            if length(inan)<2
                W12_filt2{i}(:,j) = NaN(size(W12_filt2{i}(:,j)));
            else
                W12_filt2{i}(:,j) = interp1(inan,W12_filt2{i}(inan,j),1:size(W12_filt2{i},1),interp_method,NaN);
            end
        end

        if ~isempty(W22)
            inan = find(~isnan(W22_filt2{i}(:,j)));
            if length(inan)<2
                W22_filt2{i}(:,j) = NaN(size(W22_filt2{i}(:,j)));
            else
                W22_filt2{i}(:,j) = interp1(inan,W22_filt2{i}(inan,j),1:size(W22_filt2{i},1),interp_method,NaN);
            end
        end
    end
end

%% Calculate mean and standard deviation of velocity components, after filtering/interpolation
BasicStatsUnfiltered2 = struct('VelMean',[],'VelStd',[]);
for i=1:length(U2)
    for n=1:size(U2{1},2)
        BasicStatsUnfiltered2.VelMean(i,n,1)=mean(U2_filt2{i}(:,n),'omitnan');
        BasicStatsUnfiltered2.VelStd(i,n,1)=std(U2_filt2{i}(:,n),'omitnan');
        if ~isempty(V2)
            BasicStatsUnfiltered2.VelMean(i,n,2)=mean(V2_filt2{i}(:,n),'omitnan');
            BasicStatsUnfiltered2.VelStd(i,n,2)=std(V2_filt2{i}(:,n),'omitnan');
        end
        if ~isempty(W12)
            BasicStatsUnfiltered2.VelMean(i,n,3)=mean(W12_filt2{i}(:,n),'omitnan');
            BasicStatsUnfiltered2.VelStd(i,n,3)=std(W12_filt2{i}(:,n),'omitnan');
        end
        if ~isempty(W22)
            BasicStatsUnfiltered2.VelMean(i,n,4)=mean(W22_filt2{i}(:,n),'omitnan');
            BasicStatsUnfiltered2.VelStd(i,n,4)=std(W22_filt2{i}(:,n),'omitnan');
        end
    end
end

%% plot mean and standard deviation of velocities (both with and without filtering/interpolation
if figSwitchPP==1
    figure;
    subplot(4,1,1);
    E11 = errorbar(BasicStatsUnfiltered.VelMean(:,1,1),BasicStatsUnfiltered.VelStd(:,1,1),'.');
    ylabel('u');
    hold on;
    E12 = errorbar(BasicStatsUnfiltered2.VelMean(:,1,1),BasicStatsUnfiltered2.VelStd(:,1,1),'o');
    plot(xlim,[0 0],'--k');
    hold off;

    if ~isempty(V2)
        subplot(4,1,2)
        E21 = errorbar(BasicStatsUnfiltered.VelMean(:,1,2),BasicStatsUnfiltered.VelStd(:,1,2),'.');
        ylabel('v');
        hold on;
        E22 = errorbar(BasicStatsUnfiltered2.VelMean(:,1,2),BasicStatsUnfiltered2.VelStd(:,1,2),'o');
        plot(xlim,[0 0],'--k');
        hold off;
    end

    if ~isempty(W12)
        subplot(4,1,3)
        E31 = errorbar(BasicStatsUnfiltered.VelMean(:,1,3),BasicStatsUnfiltered.VelStd(:,1,3),'.');
        ylabel('w1');
        hold on;
        E32 = errorbar(BasicStatsUnfiltered2.VelMean(:,1,3),BasicStatsUnfiltered2.VelStd(:,1,3),'o');
        plot(xlim,[0 0],'--k');
        hold off;
    end

    if ~isempty(W22)
        subplot(4,1,4)
        E41 = errorbar(BasicStatsUnfiltered.VelMean(:,1,4),BasicStatsUnfiltered.VelStd(:,1,4),'.');
        ylabel('w2');
        hold on;
        E42 = errorbar(BasicStatsUnfiltered2.VelMean(:,1,4),BasicStatsUnfiltered2.VelStd(:,1,4),'o');
        plot(xlim,[0 0],'--k');
        hold off;
    end
end
    
%% Show the data for visual inspection of the quality.
if figSwitchPP==1
    figure;
    subplot(4,1,1)
    hold on;
    for i=1:length(U2)
        plot(tVEL2{i},U2_filt2{i}(:,n_vertcell))
    end
    plot(xlim,[0 0],'--k');
    xlabel('time')
    ylabel('u (m/s)')
    dateaxis('x',15)
    box on;
    title(sprintf('Vertical cell %d',n_vertcell));
    hold off;

    if ~isempty(V2)
        subplot(4,1,2)
        hold on;
        for i=1:length(V2)
            plot(tVEL2{i},V2_filt2{i}(:,n_vertcell))
        end
        plot(xlim,[0 0],'--k');
        xlabel('time')
        ylabel('v (m/s)')
        dateaxis('x',15)
        box on;
        hold off;
    end

    if ~isempty(W12)
        subplot(4,1,3)
        hold on;
        for i=1:length(W12)
            plot(tVEL2{i},W12_filt2{i}(:,n_vertcell))
        end
        plot(xlim,[0 0],'--k');
        xlabel('time')
        ylabel('w1 (m/s)')
        dateaxis('x',15)
        box on;
        hold off;
    end

    if ~isempty(W22)
        subplot(4,1,4)
        hold on;
        for i=1:length(W22)
            plot(tVEL2{i},W22_filt2{i}(:,n_vertcell))
        end
        plot(xlim,[0 0],'--k');
        xlabel('time')
        ylabel('w2 (m/s)')
        dateaxis('x',15)
        box on;
        hold off;
    end
end

%% make velocity spectra
if ~isempty(n_spectra)
m=2^12;
dt=1/50;
for i=1:length(n_spectra)
    N=n_spectra(i);
    figure;
    subplot(2,2,1);
    [Pxx, F] = spec(U2_filt2{N}(~isnan(U2_filt2{N}(:,1)),1),m,dt,m/2);
    loglog(F, Pxx(:,1));
    hold on;
    plot([1 10],10^(-3.5)*[1 10].^(-5/3));
    title(sprintf('u_{%d}',N));
    ylabel('top');
    
    if ~isempty(V2)
        subplot(2,2,2);
        [Pyy, F] = spec(V2_filt2{N}(~isnan(V2_filt2{N}(:,1)),1),m,dt,m/2);
        loglog(F, Pyy(:,1));
        hold on;
        plot([1 10],10^(-3.5)*[1 10].^(-5/3));
        title(sprintf('v_{%d}',N));
        
        subplot(2,2,4);
        [Pyy, F] = spec(V2_filt2{N}(~isnan(V2_filt2{N}(:,end)),end),m,dt,m/2);
        loglog(F, Pyy(:,1));
        hold on;
        plot([1 10],10^(-3.5)*[1 10].^(-5/3));
    end

    subplot(2,2,3);
    [Pxx, F] = spec(U2_filt2{N}(~isnan(U2_filt2{N}(:,end)),end),m,dt,m/2);
    loglog(F, Pxx(:,1));
    hold on;
    plot([1 10],10^(-3.5)*[1 10].^(-5/3));
    ylabel('bottom');


end
end

%%
if isempty(V2)
    V2_filt2 = [];
end
if isempty(W12)
    W12_filt2 = [];
end
if isempty(W22)
    W22_filt2 = [];
end
