function [Npoints,U2,V2,W12,W22,tVEL2,Cor2,Xpos2,Ypos2,Zpos2,bed2,tBED2] = VectrinoProfiler_read_data_20180419(filenameADV,experiment,Nn,Nm,Np,summertime,figSwitch,drybed,filenameADV2)
% This script is meant to estimate the discharge from a series of flow
% velocity measurements with a Vectrino Profiler. Following input
% parameters should be checked before running:
% --> filenameADV
% --> experiment
% --> Nn
% --> Nm
% --> Np
% --> summertime
% --> dtPerPoint
% --> date_start
% --> 4 correction parameters for X,Y,Z en WL
% --> cellsize and distance between probe and centre of first measurement cell
% Moreover, some data should be imported beforehand (see lines 26-38).
%
% NOTE: the bed level reading from the Vectrino Profiler is in the
% calculations not corrected to the conventions of a zero-reading at the
% flume bottom.

% filenameADV = '~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/Exp%s/Vectrino/velocity_points/VectrinoData.%03d.18.Vectrino Profiler.%05d_%d.mat';
% experiment = 'C1';
% Nm = 1; % five-digit number specifying the Vectrino data-file
% Np = 40; % number specifying number of sub-files of one large Vectrino measurement
% dY = 100; % cell size in y-direction (in mm)
% summertime = 1; % 1 meaning summer time (UTC+2), 0 meaning winter time (UTC+1)
% dtPerPoint = 180; % time duration per point (in s)
% figSwitch = 1; % figSwitch = 1: create example plot of velocities; figSwitch = 0: do not create plot

correctionY = 72; % y = yADV + correctionY (later in script translated to y=2528.4-yADV=2600-72-yADV)
correctionX = 34;%41; % x = xADV + correctionX
correctionZ = -751;%-765.8;%-759.49; % z = -(zADV + z0ADV + correctionZ)
correctionWL = 58.4; % waterlevel = WL + correctionWL

% cellsize = 1; %mm
% cellstart = 40.5; %mm

% drybed = load('~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/ExpC1/laser_scanner/dry_scan/dry_scan_20180604.mat','XI','YI','VI');
% XI = C1_dry.XI{1}; %variables for bed level background of sample velocity plot
% YI = C1_dry.YI{1};
% VI = C1_dry.VI{1};

%%
% % filenameWL = '~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/ExpA2/waterlevel/lin_sens_20180227.txt';
% WL_data = importdata(filenameWL,' ',4);
% Qw = WL_data.data(:,2);
% WL = WL_data.data(:,[7:10 12:15]); % 111 112 113 114 121 122 123 124
% Qs = WL_data.data(:,11);
% time_WL = datetime(strcat(WL_data.textdata(5:end,1),{' '},WL_data.textdata(5:end,2)),'InputFormat','dd-MM-yyyy HH:mm:ss');

%%
% waterlevel = WL + correctionWL;
% time_WL_num = datenum(time_WL);
% 
% waterlevelQ = waterlevel(:,8); %which water level point to use for discharge calculation

%%
% filenameADV2 = '~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/ExpC1/Vectrino/velocity_points/Vel_06_06_2018.dat';
[timeStarttot,Xpostot,Ypostot,z0ADVtot,DLYtot,timeEndtot] = VectrinoProfiler_read_path_20180420(filenameADV2,summertime);

dum_ind = Ypostot>=0 & Xpostot>=0;%Ypostot<1600 & Xpostot==2000;
dum_ind = dum_ind(1:length(timeEndtot));

DLY = DLYtot(dum_ind);
timeEndNum = timeEndtot(dum_ind);
z0ADV = z0ADVtot(dum_ind);
timeStartNum = timeStarttot(dum_ind);
Xpos = Xpostot(dum_ind);
Ypos = Ypostot(dum_ind);

clearvars timeStarttot Xpostot Ypostot z0ADVtot DLYtot timeEndtot

%%
Xpos2 = Xpos + correctionX;
Ypos2 = 2600 - (Ypos + correctionY); %negative because of flipping of y-axis
Zpos2 = -z0ADV - correctionZ;

timeEndNum(~strcmp(DLY,'DLY100')) = [];
timeStartNum(~strcmp(DLY,'DLY100')) = [];
Xpos2(~strcmp(DLY,'DLY100')) = [];
Ypos2(~strcmp(DLY,'DLY100')) = [];
Zpos2(~strcmp(DLY,'DLY100')) = [];

%%
ConfigTot = cell(Np,1);
U = []; % along-flume velocity (m/s)
V = []; % cross-stream velocity (m/s)
W1 = []; % first estimate of vertical velocity (m/s)
W2 = []; % second estimate of vertical velocity (m/s) (for details, see correlation matrix Config.ProbeCalibration_calibrationMatrix(cell,:),4,4)' (with with cell the vertical cell number)
Cor = []; % correlation (dB)
Amp = []; % amplitude (dB)
SNR = []; % signal-to-noise ratio (dB)
tVEL = []; % time of velocity measurements (Matlab serial date number)
tBED = []; % time of bed level measurements (Matlab serial date number)
AmpBED = []; % amplitude of bed level measurements, with over 3rd dimension: CenterBeamAmp, CenterBeamCurveFit, CenterBeamBottomPeak (dB)
bed = []; % bottom distance (m)

for i=1:Np
    load(sprintf(filenameADV,experiment,Nn,Nm,i));
    ConfigTot{i} = Config;
    U = [U; -Data.Profiles_VelY];
    V = [V; Data.Profiles_VelX];
    W1 = [W1; Data.Profiles_VelZ1];
    W2 = [W2; Data.Profiles_VelZ2];
    Cor = cat(1,Cor,cat(3,Data.Profiles_CorBeam1,Data.Profiles_CorBeam2,Data.Profiles_CorBeam3,Data.Profiles_CorBeam4));
    Amp = cat(1,Amp,cat(3,Data.Profiles_AmpBeam1,Data.Profiles_AmpBeam2,Data.Profiles_AmpBeam3,Data.Profiles_AmpBeam4));
    SNR = cat(1,SNR,cat(3,Data.Profiles_SNRBeam1,Data.Profiles_SNRBeam2,Data.Profiles_SNRBeam3,Data.Profiles_SNRBeam4));
    tVEL = [tVEL; Data.Profiles_HostTimeMatlab(:)];
    tBED = [tBED; Data.BottomCheck_HostTimeMatlab(:)];
    AmpBED = cat(1,AmpBED,cat(3,Data.BottomCheck_CenterBeamAmp,Data.BottomCheck_CenterBeamCurveFit,Data.BottomCheck_CenterBeamBottomPeak));
    bed = [bed; Data.BottomCheck_BottomDistance(:)];
end

clearvars Data Config

%%
Npoints = length(Ypos2);
U2 = cell(Npoints,1);
V2 = U2;
W12 = U2;
W22 = U2;
Cor2 = U2;
Amp2 = U2;
SNR2 = U2;
tVEL2 = U2;
tBED2 = U2;
AmpBED2 = U2;
bed2 = U2;

for i=1:Npoints
    indCell = find(tVEL>=timeStartNum(i) & tVEL<=timeEndNum(i));
    tVEL2{i} = tVEL(indCell);
    U2{i} = U(indCell,:);
    V2{i} = V(indCell,:);
    W12{i} = W1(indCell,:);
    W22{i} = W2(indCell,:);
    Cor2{i} = Cor(indCell,:,:);
    Amp2{i} = Amp(indCell,:,:);
    SNR2{i} = SNR(indCell,:,:);
    
    indCellBED = find(tBED>=timeStartNum(i) & tBED<=timeEndNum(i));
    tBED2{i} = tBED(indCellBED);
    AmpBED2{i} = AmpBED(indCellBED,:,:);
    bed2{i} = bed(indCellBED);
end

clearvars tVEL U V W1 W2 Cor Amp SNR tBED AmpBED bed

%% Example plot for velocity measurements of top (blue) and bottom (red) cell
if figSwitch==1
    scale = 5000;
    figure;
    hBack = pcolor(drybed.XI{1},drybed.YI{1},drybed.VI{1});
    set(hBack,'FaceColor','interp','LineStyle','none');
    set(gca,'XDir','reverse','YDir','reverse');
    colormap(gray);
    hold on;
    axis(gca,'equal');
    xlim([0 12e3]);
    ylim([0 2.6e3]);
    set(gca,'XDir','reverse','YDir','reverse');
    for i=1:Npoints
        quiver(Xpos2(i),Ypos2(i),mean(U2{i}(:,1)),mean(V2{i}(:,1)),scale,'-b');
    end
    for i=1:Npoints
        quiver(Xpos2(i),Ypos2(i),mean(U2{i}(:,end)),mean(V2{i}(:,end)),scale,'-r');
    end
    hold off;
end

%%
% % % dumWL1 = find(time_WL_num >= timeStartNum(1) & time_WL_num <= timeEndNum(end),1);
% % % dumWL2 = find(time_WL_num >= timeStartNum(1) & time_WL_num <= timeEndNum(end),1,'last');
% % % if ~isempty(dumWL1) && ~isempty(dumWL2)
% % %     waterlevel = mean(waterlevel(dumWL1:dumWL2));
% % % elseif ~isempty(dumWL1) && isempty(dumWL2)
% % %     waterlevel = mean(waterlevel(dumWL1:end));
% % %     disp('The waterlevel is calculated over a shorter period.')
% % % else
% % %     waterlevel = mean(waterlevel(end-59):waterlevel(end));
% % %     disp('The waterlevel is calculated using the last minute of the file, because the exact timeslot was not available.')
% % % end
% % % 
% % % upper_size = waterlevel - (Zpos2 - cellstart + cellsize/2);
% % % 
% % % %%%%%%%%%%
% % % % Filter out velocity data at fixed positions.
% % % %%%
% % % 
% % % Y = nan(size(tVEL));
% % % for i=1:length(Ypos2)
% % %     Y(tVEL>=timeStartNum(i) & tVEL<=timeEndNum(i)) = Ypos2(i);
% % % end
% % % 
% % % Y_unique = unique(Y);
% % % Y_unique = Y_unique(~isnan(Y_unique));
% % % 
% % % if max(Y_unique) == Y_unique(1)
% % %     Y_unique = flipud(Y_unique);
% % % elseif min(Y_unique) ~= Y_unique(1)
% % %     prompt = input('The order of measured Y-values is different than expected. This may lead to unexpected outcomes. Do you wish to proceed? Y/N [N] ','s');
% % %     if ~strcmp(prompt,'Y') && ~strcmp(prompt,'y') && ~strcmp(prompt,'yes') && ~strcmp(prompt,'Yes')
% % %         error('Script terminated by user.');
% % %     end
% % % end
% % % 
% % % U_relevant = U(~isnan(Y),:);
% % % Y_relevant = Y(~isnan(Y));
% % % 
% % % U_relevant_mean_zeros = [];
% % % for i=1:length(Y_unique)
% % %     U_relevant_mean_zeros(i,:) =  mean(U_relevant((i-1)*dtPerPoint*ConfigTot{1}.sampleRate+51:i*dtPerPoint*ConfigTot{1}.sampleRate,:),1);
% % % end
% % % U_relevant_mean_zeros(U_relevant_mean_zeros < 0.02) = 0;
% % % 
% % % %%%%%%%%%%
% % % % Filter out bed level data at fixed profiles
% % % %%%
% % % 
% % % Yb = nan(size(tBED));
% % % for i=1:length(Ypos2)
% % %     Yb(tBED>=timeStartNum(i) & tBED<=timeEndNum(i)) = Ypos2(i);
% % % end
% % % 
% % % bed_relevant = bed(~isnan(Yb),:);
% % % % Yb_relevant = Yb(~isnan(Yb));
% % % 
% % % bed_relevant_mean = [];
% % % for i=1:length(Y_unique)
% % %     bed_relevant_mean(i,:) =  mean(bed_relevant((i-1)*dtPerPoint*ConfigTot{1}.bottom_sampleRate+11:i*dtPerPoint*ConfigTot{1}.bottom_sampleRate,:),1);
% % % end
% % % 
% % % %%%%%%%%%%
% % % 
% % % discharge = 0;
% % % for i=1:size(U_relevant_mean_zeros,1)
% % %     for j=1:size(U_relevant_mean_zeros,2)
% % %         discharge = discharge + U_relevant_mean_zeros(i,j)*dY/1000*cellsize/1000;
% % %     end
% % %     discharge = discharge + U_relevant_mean_zeros(i,1)*dY/1000*upper_size(i)/1000;
% % % end
% % % 
% % % %%%%%%%%%%
% % % 
% % % for i=1:size(bed_relevant_mean,1)
% % %     if bed_relevant_mean(i)>(cellstart+cellsize*29)/1000
% % %         discharge = discharge + 0.5*(bed_relevant_mean(i)-(cellstart+cellsize*29.5)/1000)*U_relevant_mean_zeros(i,end)*dY/1000;
% % %     end
% % % end
% % % 
% % % for i=1:size(U_relevant_mean_zeros,1)
% % %     slope(i) = -min(mean(diff(U_relevant_mean_zeros(i,1:4))),0)/(4*cellsize/1000);
% % %     discharge = discharge + 0.5*slope(i)*U_relevant_mean_zeros(i,1)*upper_size(i)/1000*dY/1000;
% % % end
% % % 
% % % discharge = discharge * 1000