function [timeStartNum,Xpostot,Ypostot,z0ADVtot,DLYtot,timeEndNum] = VectrinoProfiler_read_path_20180420(filename, summertime, startRow, endRow)
%IMPORTFILE Import numeric data from a text file as column vectors.
%   [TIMESTARTTOT,XPOSTOT,YPOSTOT,Z0ADVTOT,DLYTOT,TIMEENDTOT] =
%   IMPORTFILE(FILENAME) Reads data from text file FILENAME for the default
%   selection.
%
%   [TIMESTARTTOT,XPOSTOT,YPOSTOT,Z0ADVTOT,DLYTOT,TIMEENDTOT] =
%   IMPORTFILE(FILENAME, STARTROW, ENDROW) Reads data from rows STARTROW
%   through ENDROW of text file FILENAME.
%
% Example:
%   [timeStarttot,Xpostot,Ypostot,z0ADVtot,DLYtot,timeEndtot] = importfile('Vectrino_discharge.dat',8, 139);
%
%    See also TEXTSCAN.

% Auto-generated by MATLAB on 2018/04/20 17:02:53

%% Initialize variables.
delimiter = {'\t',' '};
if nargin<=3
    startRow = 8;
    endRow = inf;
end

%% Format string for each line of text:
%   column1: datetimes (%{HH:mm:ss,S}D)
%	column2: double (%f)
%   column3: double (%f)
%	column4: double (%f)
%   column5: text (%s)
%	column6: datetimes (%{HH:mm:ss,S}D)
% For more information, see the TEXTSCAN documentation.
formatSpec = '%{HH:mm:ss,S}D%f%f%f%s%{HH:mm:ss,S}D%[^\n\r]';

%% Open the text file.
fileID = fopen(filename,'r');

%% Read the date of start of measurements
dayRead = textscan(fileID, '%*s%f%s%f', 1, 'Delimiter', delimiter, 'MultipleDelimsAsOne', true, 'ReturnOnError', false);
date = today;
dayNumber = dayRead{1};
monthString = dayRead{2}{1};
yearNumber = dayRead{3};
switch monthString
    case 'jan'
        monthNumber = 1;
    case 'feb'
        monthNumber = 2;
    case 'mrt'
        monthNumber = 3;
    case 'apr'
        monthNumber = 4;
    case 'mei'
        monthNumber = 5;
    case 'jun'
        monthNumber = 6;
    case 'jul'
        monthNumber = 7;
    case 'aug'
        monthNumber = 8;
    case 'sep'
        monthNumber = 9;
    case 'okt'
        monthNumber = 10;
    case 'nov'
        monthNumber = 11;
    case 'dec'
        monthNumber = 12;
end

date_start = datenum(yearNumber,monthNumber,dayNumber);

%% Read columns of data according to format string.
% This call is based on the structure of the file used to generate this
% code. If an error occurs for a different file, try regenerating the code
% from the Import Tool.

%frewind(fileID);
textscan(fileID, '%[^\n\r]', startRow(1)-2, 'ReturnOnError', false); %startRow(1)-1
dataArray = textscan(fileID, formatSpec, endRow(1)-startRow(1)+1, 'Delimiter', delimiter, 'MultipleDelimsAsOne', true, 'ReturnOnError', false);
for block=2:length(startRow)
    frewind(fileID);
    textscan(fileID, '%[^\n\r]', startRow(block)-1, 'ReturnOnError', false);
    dataArrayBlock = textscan(fileID, formatSpec, endRow(block)-startRow(block)+1, 'Delimiter', delimiter, 'MultipleDelimsAsOne', true, 'ReturnOnError', false);
    for col=1:length(dataArray)
        dataArray{col} = [dataArray{col};dataArrayBlock{col}];
    end
end

%% Close the text file.
fclose(fileID);

%% Post processing for unimportable data.
% No unimportable data rules were applied during the import, so no post
% processing code is included. To generate code which works for
% unimportable data, select unimportable cells in a file and regenerate the
% script.

%% Allocate imported array to column variable names
timeStarttot = dataArray{:, 1};
Xpostot = dataArray{:, 2};
Ypostot = dataArray{:, 3};
z0ADVtot = dataArray{:, 4};
DLYtot = dataArray{:, 5};
timeEndtot = dataArray{:, 6};

% For code requiring serial dates (datenum) instead of datetime, uncomment
% the following line(s) below to return the imported dates as datenum(s).

timeEndNum = datenum(timeEndtot) - (date-date_start) - (1+summertime)/24;
timeStartNum = datenum(timeStarttot) - (date-date_start) - (1+summertime)/24;

for i=1:length(timeEndNum)-1
    if timeEndNum(i+1)<timeEndNum(i)
        timeEndNum(i+1) = timeEndNum(i+1)+1;
    end
    if timeStartNum(i+1)<timeStartNum(i)
        timeStartNum(i+1) = timeStartNum(i+1)+1;
    end
end